div.output-content,
div.shinylive-wrapper {
  background-color: rgba(250, 250, 250, 0.65);
  border: 1px solid rgba(233, 236, 239, 0.65);
  border-radius: 0.5rem;
  box-shadow: 0px 1px 2px rgba(0, 0, 0, 0.04), 0px 3px 7px rgba(0, 0, 0, 0.04),
    0px 12px 30px rgba(0, 0, 0, 0.07);
  margin-top: 32px;
  margin-bottom: 32px;
}

div.shinylive-wrapper {
  margin: 1em 0;
  border-radius: 8px;
}

.shinylive-container {
  background-color: #eeeff2;
  min-height: auto;
}

.shinylive-container > div {
  box-shadow: none;
}

.editor-container .cm-editor .cm-scroller {
  font-size: 13px;
  line-height: 1.5;
}

iframe.app-frame {
  /* Override the default margin from Bootstrap */
  margin-bottom: 0;
}
