$bslib-preset-type: bootswatch;
$bslib-preset-name: cerulean;
@import "../../bslib/lib/bsw5/dist/cerulean/_variables.scss";
$navbar-light-bg: $primary !default;
$navbar-dark-bg: $info !default;
$enable-cssgrid: true !default;
@import "../../bslib/bs3compat/_defaults.scss";
@import "../../bslib/lib/bs5/scss/_variables.scss";
@import "../../bslib/lib/bs5/scss/_variables-dark.scss";
$bootstrap-version: 5;
$bslib-preset-name: null !default;
$bslib-preset-type: null !default;
@import "../../bslib/lib/bs5/scss/_maps.scss";
@import "../../bslib/lib/bs5/scss/_mixins.scss";
@import "../../bslib/bs3compat/_declarations.scss";
