# Generated by scripts/htmlDependencies.R: do not edit by hand

from __future__ import annotations

from typing import Literal

BusySpinnerType = Literal[
    "bars",
    "bars2",
    "bars3",
    "dots",
    "dots2",
    "dots3",
    "pulse",
    "pulse2",
    "pulse3",
    "ring",
    "ring2",
    "ring3",
]
