// United 5.3.1
// Bootswatch


// Variables

$web-font-path: "https://fonts.googleapis.com/css2?family=Ubuntu:wght@400;700&display=swap" !default;
@if $web-font-path {
  @import url($web-font-path);
}

// Indicators

.badge {
  &.bg-light {
    color: $body-color;
  }
}
