:root {
--bslib-bootstrap-version: #{$bootstrap-version};
--bslib-preset-name: #{$bslib-preset-name};
--bslib-preset-type: #{$bslib-preset-type};
}
@import "../../bslib/lib/bs5/scss/mixins/_banner.scss";
@include bsBanner('')
@import "../../bslib/lib/bs5/scss/_utilities.scss";
@import "../../bslib/lib/bs5/scss/_root.scss";
@import "../../bslib/lib/bs5/scss/_reboot.scss";
@import "../../bslib/lib/bs5/scss/_type.scss";
@import "../../bslib/lib/bs5/scss/_images.scss";
@import "../../bslib/lib/bs5/scss/_containers.scss";
@import "../../bslib/lib/bs5/scss/_grid.scss";
@import "../../bslib/lib/bs5/scss/_tables.scss";
@import "../../bslib/lib/bs5/scss/_forms.scss";
@import "../../bslib/lib/bs5/scss/_buttons.scss";
@import "../../bslib/lib/bs5/scss/_transitions.scss";
@import "../../bslib/lib/bs5/scss/_dropdown.scss";
@import "../../bslib/lib/bs5/scss/_button-group.scss";
@import "../../bslib/lib/bs5/scss/_nav.scss";
@import "../../bslib/lib/bs5/scss/_navbar.scss";
@import "../../bslib/lib/bs5/scss/_card.scss";
@import "../../bslib/lib/bs5/scss/_accordion.scss";
@import "../../bslib/lib/bs5/scss/_breadcrumb.scss";
@import "../../bslib/lib/bs5/scss/_pagination.scss";
@import "../../bslib/lib/bs5/scss/_badge.scss";
@import "../../bslib/lib/bs5/scss/_alert.scss";
@import "../../bslib/lib/bs5/scss/_progress.scss";
@import "../../bslib/lib/bs5/scss/_list-group.scss";
@import "../../bslib/lib/bs5/scss/_close.scss";
@import "../../bslib/lib/bs5/scss/_toasts.scss";
@import "../../bslib/lib/bs5/scss/_modal.scss";
@import "../../bslib/lib/bs5/scss/_tooltip.scss";
@import "../../bslib/lib/bs5/scss/_popover.scss";
@import "../../bslib/lib/bs5/scss/_carousel.scss";
@import "../../bslib/lib/bs5/scss/_spinners.scss";
@import "../../bslib/lib/bs5/scss/_offcanvas.scss";
@import "../../bslib/lib/bs5/scss/_placeholders.scss";
@import "../../bslib/lib/bs5/scss/_helpers.scss";
@import "../../bslib/lib/bs5/scss/utilities/_api.scss";
.table th[align=left] { text-align: left; }
.table th[align=right] { text-align: right; }
.table th[align=center] { text-align: center; }
@import "../../bslib/bs3compat/_rules.scss";
@import "../../bslib/bslib-scss/bslib.scss";
@import "../../bslib/lib/bsw5/dist/cerulean/_bootswatch.scss";
.btn-default:not(.btn-primary):not(.btn-info):not(.btn-success):not(.btn-warning):not(.btn-danger):not(.btn-dark):not(.btn-light):not([class*='btn-outline-']) {
          @extend .btn-secondary !optional;
        }
@import "../../bslib/components/scss/mixins/_mixins.scss";
@import "../../bslib/components/scss/accordion.scss";
@import "../../bslib/components/scss/card.scss";
@import "../../bslib/components/scss/grid.scss";
@import "../../bslib/components/scss/nav_spacer.scss";
@import "../../bslib/components/scss/page_fillable.scss";
@import "../../bslib/components/scss/page_navbar.scss";
@import "../../bslib/components/scss/page_sidebar.scss";
@import "../../bslib/components/scss/sidebar.scss";
@import "../../bslib/components/scss/value_box.scss";
@import "../../shiny/www/shared/shiny_scss/shiny.bootstrap5.scss";
@import "../../shiny/www/shared/shiny_scss/shiny.scss";
@import "../../shiny/www/shared/selectize/scss/selectize.bootstrap5.scss";
$accent: $component-active-bg;
@import "../../shiny/www/shared/ionrangeslider/scss/shiny.scss";
@import "../../shiny/www/shared/datepicker/scss/build3.scss";
