@keyframes py-shiny-spin {
  0% {
    transform: rotate(0deg);
  }
  100% {
    transform: rotate(360deg);
  }
}

@media (prefers-reduced-motion: no-preference) {
  .py-shiny-spin {
    animation: py-shiny-spin 2s linear infinite;
  }
}
