.#{$selectize}-dropdown.plugin-dropdown_header {
  .#{$selectize}-dropdown-header {
    position: relative;
    padding: ($select-padding-dropdown-item-y * 2)
      $select-padding-dropdown-item-x;
    border-bottom: 1px solid $select-color-border;
    background: RGBA($select-color-dropdown, 0.15);
    border-radius: $select-border-radius $select-border-radius 0 0;
  }
  .#{$selectize}-dropdown-header-close {
    position: absolute;
    right: $select-padding-dropdown-item-x;
    top: 50%;
    color: $select-color-text;
    opacity: 0.4;
    margin-top: -12px;
    line-height: 20px;
    font-size: 20px !important;
    text-decoration: none;
  }
  .#{$selectize}-dropdown-header-close:hover {
    color: RGB($select-color-rgb);
  }
}
